<?php

    set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__) ."/../../phpsnmp");
    require_once('snmp.php');

    class Send_SNMP_Trap_Action extends Reactor_Action
    {
        protected $num_varbinds = 4;
        protected $tpl = 'send_snmp_trap.php';
        protected $name = 'Send an SNMP trap';
		
		public static $registration = array(
			ACTION_NAME => 'Send an SNMP trap message',
			ACTION_CATEGORY => 'Network',
			ACTION_DESCRIPTION => 'Send an SNMP trap to a designated destination',
			ACTION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '01/22/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array(),	//Misc information	
		); 		
		
        protected $fields = array(
            'manager' => array(
                'label' => 'manager',
                'validate' => 'trim|required|xss_clean|callback_check_hostname',
            ),
            'snmp_version' => array(
                'label' => 'SNMP version',
                'validate' => 'trim|required',
            ),
            'community' => array(
                'label' => 'SNMP community',
                'validate' => 'trim|required',
            ),
            'enterprise_oid' => array(
                'label' => 'enterprise oid',
                'validate' => 'trim|callback_check_oid',
            ),
            'trap_type' => array(
                'label' => 'trap type',
                'validate' => 'trim'
            ),
            'specific_trap_type' => array(
                'label' => 'specific trap type',
                'validate' => 'trim|numeric'
            ),
        );

        public function __construct($params=FALSE, $event_chain_run=FALSE) {
            // varbinds
            for($i=0; $i<$this->num_varbinds; $i++) {
                $this->fields['var_type'.$i] = array(
                    'label' => 'type '.$i,
                    'validate' => 'trim'
                );
                $this->fields['var_oid'.$i] = array(
                    'label' => 'oid '.$i,
                    'validate' => 'trim|callback_check_oid'
                );
                $this->fields['var_val'.$i] = array(
                    'label' => 'value '.$i,
                    'validate' => 'trim'
                );
            }
            parent::__construct($params, $event_chain_run);
        }

        /**
         * Form validation to check an OID value
         **/
        public function check_oid($str)
        {
            if(!strlen($str)) { return TRUE; }
            $oid_expression = '/^([0-9\.]+\.)+([0-9]+)$/';
            if(!preg_match($oid_expression, $str)) {
                $this->ci->form_validation->set_message('check_oid', 'The OID specified is invalid.');
                return FALSE;
            } else {
                return TRUE;
            }
        }

        /** 
         * Form validation function to check a hostname.
         **/
        public function check_hostname($str)
        {
            $hostname_expression = '/^(?=.{1,255}$)[0-9A-Za-z]'.
                                   '(?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?'.
                                   '(?:\.[0-9A-Za-z](?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?)*\.?$/i';
            if(!preg_match($hostname_expression, $str)) {
                $this->ci->form_validation->set_message('check_hostname', 'The hostname specified is invalid.');
                return FALSE;
            } else {
                return TRUE;
            }
        }

        public function get_label()
        {
            if($params = $this->get_params()) {
                return sprintf('Send an SNMP trap to \'%s\'', $params['manager']);
            }
            return parent::get_label();
        }
       
        public function run($context)
        {
            $this->set_context($context);
            $this->log('info', sprintf('Sending SNMP trap packet to %s', $this->get_param('manager')));
            $snmp = new snmp();
            $snmp->version = $this->get_param('snmp_version', SNMP_VERSION_2);

            // load up variable bindings
            $oids = array();
            $values = array();
            $types = array();
            for($i=0; $i<$this->num_varbinds; $i++) {
                if($oid = $this->get_param('var_oid'.$i)) {
                    $oids[] = $oid;
                    $values[] = $this->get_param('var_val'.$i);
                    $types[] = $this->get_param('var_type'.$i);
                    $varbind = $snmp->build_varbind($oids, $values, $types);
                }
            }
            $manager  = $this->get_param('manager');
            $enterprise_oid = $this->get_param('enterprise_oid'); 
            $community = $this->get_param('community');
            $trap_type = $this->get_param('trap_type');
            $specific_trap_type = $this->get_param('specific_trap_type');
            $run_trap = $snmp->trap($manager, $community, $varbind, $enterprise_oid, $agent='127.0.0.1', $trap_type, $specific_trap_type, 0);
            $this->set_result($run_trap);
            return $context;
        }
    }
